'use strict';

const fs = require('fs');
const path = require('path');
const crypto = require('crypto');

/**
 * @typedef FileInfo
 * @type {object}
 *
 */
class FileInfo {
    /**
     *
     * @returns {number}
     */
    static get FILE_NO_OPEN () {
        return -1;
    }
    /**
     * fileOptions = {
     *      directory,
     *      nameWithExtension,
     *      size
     * }
     *
     */
    constructor(fileOptions) {
        let allParts = path.parse(path.join(fileOptions.directory, fileOptions.nameWithExtension));

        this.root = allParts.root;
        this.directory = allParts.dir;
        this.baseName = allParts.base;
        this.name = allParts.name;
        this.extension = allParts.ext;

        this.size = fileOptions.size || 0;

        this._checkingFilePath = fileOptions.checkingFilePath;

        this.fd = FileInfo.FILE_NO_OPEN; //file is close (if fd > 0, then file is open)
    }

    get path() {
        return path.join(this.directory, this.baseName);
    }

    get md5() {
        let buf = fs.readFileSync(this.path);

        let hash = crypto.createHash('md5');
        hash.update(buf);

        return hash.digest('hex');
    }

    get checkingFilePathParts() {
        return this._checkingFilePath.split(path.sep);
    }

    md5Async(callback) {
        fs.readFile(this.path, (err, buf) => {
            let hash = crypto.createHash('md5');
            hash.update(buf);
            callback(err, hash.digest('hex'));
        });
    }
}

module.exports = FileInfo;
