'use strict';

const path = require('path');
const _ = require("lodash");

const FileInfo = require('../../fileSystemLayer/fileInfo');

//TODO: I am idiot. Needs use mapper libraries. Refactor.
class ApplicationsInfoBuilder {
    constructor(jsObject, saveDirectory) {
        this._jsonApplications = jsObject.applications;
        this._saveDirectory = saveDirectory;

        this._applications = [];
        this._applicationsDownloadingInfo = [];

        this._allDownloadingInfos = [];
        this._allFilesInfo = [];
    }

    buildApplications() {
        if(!this._applications) {
            this._buildAll();
        }

        return this._applications;
    }

    buildApplicationsDownloadingInfo() {
        if(!this._applicationsDownloadingInfo) {
            this._buildAll();
        }

        return this._applicationsDownloadingInfo;
    }

    buildAllDownloadingInfos() {
        if(!this._applicationsDownloadingInfo) {
            this._buildAll();
        }

        return this._allDownloadingInfos;
    }

    buildAllFilesInfo() {
        if(this._applicationsDownloadingInfo) {
            this._buildAll();
        }

        return this._allFilesInfo;
    }

    _buildAll() {
        for(let i = 0; i < this._jsonApplications.length; i++) {
            let app = this._createApplication(i);
            this._applications.push(app);

            let appDownInfo = this._createApplicationsDownloadingInfo(i);
            if(appDownInfo.downloadingInfos.length > 0) {
                this._applicationsDownloadingInfo.push(appDownInfo);
            }
        }
    }

    _createApplication(index) {
        let jsonApplicationObj = this._jsonApplications[index];

        return {
            code: `${jsonApplicationObj.id}-${jsonApplicationObj.categoryId}`,
            category: jsonApplicationObj.categoryName,
            applicationTitle: jsonApplicationObj.name,
            downloadedFilesInfo: []
        }
    }

    _createApplicationsDownloadingInfo(index) {
        let jsonApplicationObj = this._jsonApplications[index];
        let downloadingInfos = [];
        jsonApplicationObj.files.forEach((jsonFileObj) => {
            downloadingInfos.push( this._createDownloadingInfo(jsonFileObj) );
        });
        return {
            code: `${jsonApplicationObj.id}-${jsonApplicationObj.categoryId}`,
            downloadingInfos: downloadingInfos
        }
    }

    _createDownloadingInfo(jsonFileObj) {
        let directory = this._correctDirectory(jsonFileObj.filePath);
        let checkingFilePath = ApplicationsInfoBuilder._correctFilePath(jsonFileObj.filePath);
        let fileInfo = {
            directory: directory,
            nameWithExtension: ApplicationsInfoBuilder._correctName(jsonFileObj.filePath),
            size: jsonFileObj.size || 0,
            checkingFilePath
        };

        this._allFilesInfo.push(fileInfo);

        let downloadingInfo = {
            downloadUrl: jsonFileObj.downloadUrl,
            fileInfo: fileInfo
        };
        this._allDownloadingInfos.push(downloadingInfo);

        return downloadingInfo;
    }

    _correctDirectory(jsonFilePath) {
        let correctFilePath = ApplicationsInfoBuilder._correctFilePath(jsonFilePath);

        return path.join(this._saveDirectory, correctFilePath);
    }

    static _correctFilePath(filePath) {
        let filePathParts = filePath.split(/[\\\/]/);

        return filePathParts.reduce((prevValue, part, index) => {
            part = _.trimEnd(part, "."); // dot at the end of folder name can cause problems, e.g. in Windows Explorer https://stackoverflow.com/questions/4075753/how-to-delete-a-folder-that-name-ended-with-a-dot
            let separator = (index === filePathParts.length - 2) ? '' : path.sep;
            return (index === filePathParts.length - 1) ? prevValue : prevValue + part + path.sep;
        }, '');
    }

    static _correctName(jsonFilePath) {
        let filePathParts = jsonFilePath.split(/[\\\/]/);
        return filePathParts[filePathParts.length - 1];
    }
}

module.exports = ApplicationsInfoBuilder;