const { ipcRenderer } = require("electron");
const uuid = require("uuid");

const showNativeDialog = (options) => {
    return new Promise((resolve) => {
        const callbackId = "onDialogResult-" + uuid.v4();
        ipcRenderer.send("show-native-message-dialog", callbackId, options);
        ipcRenderer.once(callbackId, (_, result) => resolve(result));
    });
};

const defaultConfirmationDialogOptions = {
    title: "Are you sure?",
    type: "info",
    buttons: ["Cancel", "Ok"],
    defaultId: 0,
};

const showNativeConfirmationDialog = (message, options) => {
    showNativeDialog({
        ...defaultConfirmationDialogOptions,
        title: options?.title || defaultConfirmationDialogOptions.title,
        type: options?.type || defaultConfirmationDialogOptions.type,
        message,
        buttons: [options?.cancelButtonText || defaultConfirmationDialogOptions.buttons[0], options?.confirmButtonText || defaultConfirmationDialogOptions.buttons[1]],
    }).then((dialogResult) => {
        if (dialogResult.response == 1) {
            options.callback();
        } else if (options.cancelCallback) {
            options.cancelCallback();
        }
    });
};

const showNativeAlertDialog = (message, title, options) => {
    showNativeDialog({ message, title, type: options?.type || "info" }).then(() => {
        if (options?.callback) {
            options.callback();
        }
    });
};

module.exports.showNativeConfirmationDialog = showNativeConfirmationDialog;
module.exports.showNativeAlertDialog = showNativeAlertDialog;
