'use strict';

function bytesToSize(bytes) {
    if (bytes === 0) {
        return '0 Byte';
    }
    let k = 1024;
    let sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    let i = Math.floor(Math.log(bytes) / Math.log(k));
    return (bytes / Math.pow(k, i)).toFixed(i ? 1 : 0) + ' ' + sizes[i];
}

function bytesToMB(bytes) {
    return (bytes / Math.pow(1024, 2)).toFixed(1);
}

function msToMinutes(ms) {
    return (ms / (1000 * 60)).toFixed(1);
}

function intervalToTime (sec) {
    if (isNaN(sec) || !isFinite(sec)) {
        return '--:--:--:--';
    }
    let x = sec;
    let seconds = ('00' + parseInt(x % 60)).substr(-2);
    x /= 60;
    let minutes = ('00' + parseInt(x % 60)).substr(-2);
    x /= 60;
    let hours = ('00' + parseInt(x % 24)).substr(-2);
    x /= 24;
    let days = ('00' + parseInt(x)).substr(-2);

    return [days, hours, minutes, seconds].join(':');
}

function getFilesSize(listOfFileInfo) {
    return listOfFileInfo.reduce((prevValue, fileInfo) => {
        return prevValue + fileInfo.size;
    }, 0);
}

function getFilesSizeForJsonApplications(jsonApps) {
    let allFilesSize = 0;
    jsonApps.forEach((jsonApp) => {
        jsonApp.files.forEach((file) => {
            allFilesSize += file.size;
        });
    });

    return allFilesSize;
}

function isValidHttpUrl(value) {
    try {
        const newUrl = new URL(value);
        return newUrl.protocol === "http:" || newUrl.protocol === "https:";
    } catch {
        return false;
    }
}

module.exports.getFilesSize = getFilesSize;
module.exports.getFilesSizeForJsonApplications = getFilesSizeForJsonApplications;
module.exports.bytesToSize = bytesToSize;
module.exports.bytesToMB = bytesToMB;
module.exports.msToMinutes = msToMinutes;
module.exports.intervalToTime = intervalToTime;
module.exports.isValidHttpUrl = isValidHttpUrl;