const { ipcRenderer } = require("electron");
const EE = require("events");
const uuid = require("uuid");

class DownloadSchedulerClient extends EE {
    #started = false;
    #totalDownloadedSize = 0;
    #totalSize = 0;
    #totalFailedSize = 0;
    #totalFileCount = 0;
    #remainingFileCount = 0;
    #failedFileCount = 0;

    get started() {
        return this.#started;
    }

    get totalSize() {
        return this.#totalSize;
    }

    get totalDownloadedSize() {
        return this.#totalDownloadedSize;
    }

    get totalFailedSize() {
        return this.#totalFailedSize;
    }

    //0.01; 0.2; 0.5; min = 0; max = 1.0;
    get progressSizeInPercent() {
        return (this.#totalDownloadedSize + this.#totalFailedSize) / this.#totalSize;
    }

    get failedFileCount() {
        return this.#failedFileCount;
    }

    get downloadedFileCount() {
        return this.#totalFileCount - this.#remainingFileCount - this.#failedFileCount;
    }

    get totalFileCount() {
        return this.#totalFileCount;
    }

    get totalSize() {
        return this.#totalSize;
    }

    constructor() {
        super();

        this.on("started", (e) => {
            this.#started = true;
            this.#totalSize = e.totalSize;
            this.#totalFileCount = e.totalFileCount;
            this.#totalDownloadedSize = 0;
            this.#totalFailedSize = 0;
            this.#remainingFileCount = e.totalFileCount;
            this.#failedFileCount = 0;
        });

        this.on("progressDownloading", (e) => {
            this.#totalDownloadedSize = e.totalDownloadedSize;
        });

        this.on("endDownloadFile", (e) => {
            this.#remainingFileCount = e.remainingFileCount;
        });

        this.on("insufficientResources", (e) => {
            this.#totalDownloadedSize = e.totalDownloadedSize;
        });

        this.on("downloadError", (e) => {
            this.#totalFailedSize = e.totalFailedSize;
            this.#failedFileCount = e.failedFileCount;
            this.#remainingFileCount = e.remainingFileCount;
        });

        this.on("endDownloadAllFiles", (e) => {
            this.#started = false;
        });

        this.on("totalSizeChanged", (e) => {
            this.#totalSize = e.totalSize;
        })

        ipcRenderer.on("downloaderProcessMessage", (_, message) => {
            this.emit(message.eventType, ...message.args);
        });
    }

    stop() {
        return this.#sendMessage("stop");
    }

    #sendMessage(type, ...args) {
        const callbackId = uuid.v4();
        ipcRenderer.send("downloaderProcessMessage", type, args, callbackId);

        return new Promise((resolve) => {
            this.once(callbackId, resolve);
        });
    }
}

const downloadScheduler = new DownloadSchedulerClient();
module.exports = downloadScheduler;
